package com.googlecode.reflective

import doc.DocumentationReflection._
import doc.{Documentation, DocumentationReflection}
import java.lang.reflect.Method
import ref.SoftReference

/**
 * 
 *
 * @author Matt Hicks <mhicks@sgine.org>
 */
class EnhancedMethod protected[reflective](val parent: EnhancedClass, m: Method) {
  def name = m.getName
  lazy val args: List[MethodArgument] = for ((dc, index) <- _docs.args.zipWithIndex) yield {
    new MethodArgument(index, dc.name, dc.`type`, getDefault(index, dc.`type`), dc.doc)
  }
  def arg(name: String) = args.find(ma => ma.name == name)
  def returnType = _docs.returnClass
  def docs = _docs.docs
  def docsURL = _docs.url

  private def getDefault(index: Int, `type`: EnhancedClass) = {
    val defaultMethodName = name + "$default$" + (index + 1)
    parent.method(defaultMethodName, `type`)
  }

  def apply[R](instance: AnyRef, args: Any*) = m.invoke(instance, args.map(a => a.asInstanceOf[AnyRef]): _*).asInstanceOf[R]

  private[reflective] def argsMatch(args: Seq[EnhancedClass]) = m.getParameterTypes.zip(args).forall(t => class2EnhancedClass(t._1) == t._2)

  private lazy val _docs = parent.getDocs.method(m)

  override def toString() = {
    // public static void org.lwjgl.opengl.GL11.glVertexPointer(int,int,java.nio.DoubleBuffer)
    val b = new StringBuilder()
    b.append("def ")
    b.append(name)
    b.append('(')
    b.append(args.mkString(", "))
    b.append("): ")
    b.append(m.returnType.`type`)
    b.toString
  }
}